@@ -50,9 +50,9 @@ int Detail(char *dev, int brief, int test)
 	int is_26 = get_linux_version() >= 2006000;
 	int is_rebuilding = 0;
 	int failed = 0;
-	struct supertype *st = NULL;
 
-	void *super = NULL;
+	mdp_super_t super;
+	int have_super = 0;
 	int rv = test ? 4 : 1;
 
 	if (fd < 0) {
@@ -83,8 +83,6 @@ int Detail(char *dev, int brief, int test)
 		close(fd);
 		return rv;
 	}
-	st = super_by_version(array.major_version, array.minor_version);
-
 	if (fstat(fd, &stb) != 0 && !S_ISBLK(stb.st_mode))
 		stb.st_rdev = 0;
 	rv = 0;
@@ -101,21 +99,16 @@ int Detail(char *dev, int brief, int test)
 		    disk.minor == 0)
 			continue;
 		if ((dv=map_dev(disk.major, disk.minor))) {
-			if (!super && (disk.state & (1<<MD_DISK_ACTIVE))) {
+			if (!have_super && (disk.state & (1<<MD_DISK_ACTIVE))) {
 				/* try to read the superblock from this device
 				 * to get more info
 				 */
 				int fd2 = open(dv, O_RDONLY);
-				if (fd2 >=0 && st &&
-				    st->ss->load_super(st, fd2, &super, NULL) == 0) {
-					struct mdinfo info;
-					st->ss->getinfo_super(&info, super);
-					if (info.array.ctime != array.ctime ||
-					    info.array.level != array.level) {
-						free(super);
-						super = NULL;
-					}
-				}
+				if (fd2 >=0 &&
+				    load_super(fd2, &super) ==0 &&
+				    (unsigned long)super.ctime == (unsigned long)array.ctime &&
+				    (unsigned int)super.level == (unsigned int)array.level)
+					have_super = 1;
 				if (fd2 >= 0) close(fd2);
 			}
 		}
@@ -131,7 +124,7 @@ int Detail(char *dev, int brief, int test)
 		struct mdstat_ent *ms = mdstat_read(0);
 		struct mdstat_ent *e;
 		int devnum = array.md_minor;
-		if (MAJOR(stb.st_rdev) != MD_MAJOR)
+		if (major(stb.st_rdev) != MD_MAJOR)
 			devnum = -1 - devnum;
 
 		for (e=ms; e; e=e->next)
@@ -168,7 +161,7 @@ int Detail(char *dev, int brief, int test)
 		atime = array.utime;
 		printf("    Update Time : %.24s\n", ctime(&atime));
 		printf("          State : %s%s%s\n",
-		       (array.state&(1<<MD_SB_CLEAN))?"clean":"dirty",
+		       (array.state&(1<<MD_SB_CLEAN))?"clean":"active",
 		       array.active_disks < array.raid_disks? ", degraded":"",
 		       (!e || e->percent < 0) ? "" :
 		        (e->resync) ? ", resyncing": ", recovering");
@@ -205,8 +198,15 @@ int Detail(char *dev, int brief, int test)
 		}
 		free_mdstat(ms);
 
-		if (super && st)
-			st->ss->detail_super(super);
+		if (have_super) {
+			printf("           UUID : ");
+			if (super.minor_version >= 90)
+				printf("%08x:%08x:%08x:%08x", super.set_uuid0, super.set_uuid1,
+				       super.set_uuid2, super.set_uuid3);
+			else
+				printf("%08x", super.set_uuid0);
+			printf("\n         Events : %d.%d\n\n", super.events_hi, super.events_lo);
+		}
 
 		printf("    Number   Major   Minor   RaidDevice State\n");
 	}
@@ -251,8 +251,8 @@ int Detail(char *dev, int brief, int test)
 					 * device from the array, and then put it back.
 					 * If this fails, we are rebuilding
 					 */
-					int err = ioctl(fd, HOT_REMOVE_DISK, MKDEV(disk.major, disk.minor));
-					if (err == 0) ioctl(fd, HOT_ADD_DISK, MKDEV(disk.major, disk.minor));
+					int err = ioctl(fd, HOT_REMOVE_DISK, makedev(disk.major, disk.minor));
+					if (err == 0) ioctl(fd, HOT_ADD_DISK, makedev(disk.major, disk.minor));
 					if (err && errno ==  EBUSY)
 						printf(" rebuilding");
 				}
@@ -278,9 +278,14 @@ int Detail(char *dev, int brief, int test)
 		if (!brief) printf("\n");
 	}
 	if (spares && brief) printf(" spares=%d", spares);
-	if (super && brief && st)
-		st->ss->brief_detail_super(super);
-
+	if (have_super && brief) {
+		printf(" UUID=");
+	    	if (super.minor_version >= 90)
+			printf("%08x:%08x:%08x:%08x", super.set_uuid0, super.set_uuid1,
+			       super.set_uuid2, super.set_uuid3);
+		else
+			printf("%08x", super.set_uuid0);
+	}
 	if (brief && devices) printf("\n   devices=%s", devices);
 	if (brief) printf("\n");
 	if (test && (rv&2)) rv &= ~1;
